#!/usr/bin/env python

# Copyright 2023 Monster Street Systems LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# NB: This is a generated file; any changes will be lost.

from datetime import datetime
from enum import Enum
from sys import stderr
from typing import Optional, Set

from ulid import ULID

from vements import types
from vements.client import Client, ClientConfig


class Output(Enum):
    TEXT = 0
    OBJECT = 1


class Resource:
    _client: Client
    _output: Output
    _verbose: bool

    def __init__(self, client: Client, output: Output = Output.OBJECT, verbose: bool = False):
        self._client = client
        self._output = output
        self._verbose = verbose


class Achievement(Resource):
    def leaderboard(
        self,
        achievement_id: str,
    ):
        """Achievement leaderboard

        Reads and returns achievement leaderboard

        This is a GET operation.  It returns the response text
        or an instance of AchievementLeaderboardResponse.
        """
        path = f"/achievement/{achievement_id}/leaderboard"
        params = {}

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementLeaderboardResponse(**response.json())
        )

    def record(
        self,
        achievement_id: str,
        participant_id: ULID | str,
        value: int,
        recorded: Optional[datetime],
    ):
        """Record achievement progress

        Records and returns achievement progress

        This is a PUT operation.  It returns the response text
        or an instance of AchievementProgressResponse.
        """
        path = f"/achievement/{achievement_id}/progress"
        params = {}
        params["participant_id"] = participant_id
        params["value"] = value
        params["recorded"] = recorded

        try:
            response = self._client.put(path, params)
        except Exception as ex:
            print("client put exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client put client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementProgressResponse(**response.json())
        )

    def list(
        self,
        project_id: ULID | str,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ):
        """List achievements

        Reads and returns list of achievements in the given project

        This is a GET operation.  It returns the response text
        or an instance of AchievementListResponse.
        """
        path = f"/achievement"
        params = {}
        if project_id is not None:
            params["project_id"] = project_id
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementListResponse(**response.json())
        )

    def create(
        self,
        project_id: ULID | str,
        display: str,
        goal: int,
        repeats: int,
        locked_image: Optional[str],
        unlocked_image: Optional[str],
        position: int,
        public: bool,
        extra: Optional[object],
    ):
        """Create achievement

        Creates and returns achievement in the given project

        This is a PUT operation.  It returns the response text
        or an instance of AchievementCreateResponse.
        """
        path = f"/achievement"
        params = {}
        params["project_id"] = project_id
        params["display"] = display
        params["goal"] = goal
        params["repeats"] = repeats
        params["locked_image"] = locked_image
        params["unlocked_image"] = unlocked_image
        params["position"] = position
        params["public"] = public
        params["extra"] = extra

        try:
            response = self._client.put(path, params)
        except Exception as ex:
            print("client put exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client put client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementCreateResponse(**response.json())
        )

    def read(
        self,
        achievement_id: ULID | str,
    ):
        """Read achievement

        Reads and returns achievement by achievement id

        This is a GET operation.  It returns the response text
        or an instance of AchievementReadResponse.
        """
        path = f"/achievement/{achievement_id}"
        params = {}

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementReadResponse(**response.json())
        )

    def update(
        self,
        achievement_id: ULID | str,
        display: str,
        goal: int,
        repeats: int,
        locked_image: Optional[str],
        unlocked_image: Optional[str],
        position: int,
        public: bool,
        extra: Optional[object],
    ):
        """Update achievement

        Updates and returns achievement by achievement id

        This is a POST operation.  It returns the response text
        or an instance of AchievementUpdateResponse.
        """
        path = f"/achievement/{achievement_id}"
        params = {}
        params["display"] = display
        params["goal"] = goal
        params["repeats"] = repeats
        params["locked_image"] = locked_image
        params["unlocked_image"] = unlocked_image
        params["position"] = position
        params["public"] = public
        params["extra"] = extra

        try:
            response = self._client.post(path, params)
        except Exception as ex:
            print("client post exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client post client exception:",
                response.status_code,
                response.content,
                file=stderr,
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementUpdateResponse(**response.json())
        )

    def delete(
        self,
        achievement_id: ULID | str,
    ):
        """Delete achievement by id.

        Delete achievement by achievement id

        This is a DELETE operation.  It returns the response text
        or an instance of AchievementDeleteResponse.
        """
        path = f"/achievement/{achievement_id}"

        try:
            response = self._client.delete(path)
        except Exception as ex:
            print("client delete exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client delete client exception:",
                response.status_code,
                response.content,
                file=stderr,
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.AchievementDeleteResponse(**response.json())
        )


class Participant(Resource):
    def progress(
        self,
        participant_id: str,
    ):
        """Participant progress

        Reads and returns participant progress.

        This is a GET operation.  It returns the response text
        or an instance of ParticipantProgressResponse.
        """
        path = f"/participant/{participant_id}/progress"
        params = {}

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantProgressResponse(**response.json())
        )

    def scores(
        self,
        participant_id: str,
    ):
        """Participant scores

        Reads and returns participant scores.

        This is a GET operation.  It returns the response text
        or an instance of ParticipantScoresResponse.
        """
        path = f"/participant/{participant_id}/scores"
        params = {}

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantScoresResponse(**response.json())
        )

    def list(
        self,
        project_id: ULID | str,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ):
        """List participants

        Reads and returns list of participants in the given project

        This is a GET operation.  It returns the response text
        or an instance of ParticipantListResponse.
        """
        path = f"/participant"
        params = {}
        if project_id is not None:
            params["project_id"] = project_id
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantListResponse(**response.json())
        )

    def create(
        self,
        project_id: ULID | str,
        display: str,
        external_id: str,
        image: Optional[str],
        extra: Optional[object],
    ):
        """Create participant

        Creates and returns participant in the given project

        This is a PUT operation.  It returns the response text
        or an instance of ParticipantCreateResponse.
        """
        path = f"/participant"
        params = {}
        params["project_id"] = project_id
        params["display"] = display
        params["external_id"] = external_id
        params["image"] = image
        params["extra"] = extra

        try:
            response = self._client.put(path, params)
        except Exception as ex:
            print("client put exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client put client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantCreateResponse(**response.json())
        )

    def read(
        self,
        participant_id: ULID | str,
    ):
        """Read participant

        Reads and returns participant by participant id

        This is a GET operation.  It returns the response text
        or an instance of ParticipantReadResponse.
        """
        path = f"/participant/{participant_id}"
        params = {}

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantReadResponse(**response.json())
        )

    def update(
        self,
        participant_id: ULID | str,
        display: str,
        external_id: str,
        image: Optional[str],
        extra: Optional[object],
    ):
        """Update participant

        Updates and returns participant by participant id

        This is a POST operation.  It returns the response text
        or an instance of ParticipantUpdateResponse.
        """
        path = f"/participant/{participant_id}"
        params = {}
        params["display"] = display
        params["external_id"] = external_id
        params["image"] = image
        params["extra"] = extra

        try:
            response = self._client.post(path, params)
        except Exception as ex:
            print("client post exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client post client exception:",
                response.status_code,
                response.content,
                file=stderr,
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantUpdateResponse(**response.json())
        )

    def delete(
        self,
        participant_id: ULID | str,
    ):
        """Delete participant by id.

        Delete participant by participant id

        This is a DELETE operation.  It returns the response text
        or an instance of ParticipantDeleteResponse.
        """
        path = f"/participant/{participant_id}"

        try:
            response = self._client.delete(path)
        except Exception as ex:
            print("client delete exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client delete client exception:",
                response.status_code,
                response.content,
                file=stderr,
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ParticipantDeleteResponse(**response.json())
        )


class Scoreboard(Resource):
    def record(
        self,
        scoreboard_id: str,
        participant_id: ULID | str,
        value: int,
        recorded: Optional[datetime],
    ):
        """Record a scoreboard score

        Records and returns a scoreboard score.

        This is a PUT operation.  It returns the response text
        or an instance of ScoreboardScoreResponse.
        """
        path = f"/scoreboard/{scoreboard_id}/score"
        params = {}
        params["participant_id"] = participant_id
        params["value"] = value
        params["recorded"] = recorded

        try:
            response = self._client.put(path, params)
        except Exception as ex:
            print("client put exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client put client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardScoreResponse(**response.json())
        )

    def scores(
        self,
        scoreboard_id: str,
        from_: Optional[datetime] = None,
        to: Optional[datetime] = None,
    ):
        """Scoreboard scores

        Reads and returns scoreboard scores

        This is a GET operation.  It returns the response text
        or an instance of ScoreboardScoresResponse.
        """
        path = f"/scoreboard/{scoreboard_id}/scores"
        params = {}
        if from_ is not None:
            params["from"] = from_
        if to is not None:
            params["to"] = to

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardScoresResponse(**response.json())
        )

    def list(
        self,
        project_id: ULID | str,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ):
        """List scoreboards

        Reads and returns list of scoreboards in the given project

        This is a GET operation.  It returns the response text
        or an instance of ScoreboardListResponse.
        """
        path = f"/scoreboard"
        params = {}
        if project_id is not None:
            params["project_id"] = project_id
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardListResponse(**response.json())
        )

    def create(
        self,
        project_id: ULID | str,
        display: str,
        rank_dir: str,
        public: bool,
        extra: Optional[object],
    ):
        """Create scoreboard

        Creates and returns scoreboard in the given project

        This is a PUT operation.  It returns the response text
        or an instance of ScoreboardCreateResponse.
        """
        path = f"/scoreboard"
        params = {}
        params["project_id"] = project_id
        params["display"] = display
        params["rank_dir"] = rank_dir
        params["public"] = public
        params["extra"] = extra

        try:
            response = self._client.put(path, params)
        except Exception as ex:
            print("client put exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client put client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardCreateResponse(**response.json())
        )

    def read(
        self,
        scoreboard_id: ULID | str,
    ):
        """Read scoreboard

        Reads and returns scoreboard by scoreboard id

        This is a GET operation.  It returns the response text
        or an instance of ScoreboardReadResponse.
        """
        path = f"/scoreboard/{scoreboard_id}"
        params = {}

        try:
            response = self._client.get(path, params=params)
        except Exception as ex:
            print("client get exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client get client exception:", response.status_code, response.content, file=stderr
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardReadResponse(**response.json())
        )

    def update(
        self,
        scoreboard_id: ULID | str,
        display: str,
        rank_dir: str,
        public: bool,
        extra: Optional[object],
    ):
        """Update scoreboard

        Updates and returns scoreboard by scoreboard id

        This is a POST operation.  It returns the response text
        or an instance of ScoreboardUpdateResponse.
        """
        path = f"/scoreboard/{scoreboard_id}"
        params = {}
        params["display"] = display
        params["rank_dir"] = rank_dir
        params["public"] = public
        params["extra"] = extra

        try:
            response = self._client.post(path, params)
        except Exception as ex:
            print("client post exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client post client exception:",
                response.status_code,
                response.content,
                file=stderr,
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardUpdateResponse(**response.json())
        )

    def delete(
        self,
        scoreboard_id: ULID | str,
    ):
        """Delete scoreboard by id.

        Delete scoreboard by scoreboard id

        This is a DELETE operation.  It returns the response text
        or an instance of ScoreboardDeleteResponse.
        """
        path = f"/scoreboard/{scoreboard_id}"

        try:
            response = self._client.delete(path)
        except Exception as ex:
            print("client delete exception:", ex, file=stderr)
            return
        if response.status_code < 200 or response.status_code >= 300:
            print(
                "client delete client exception:",
                response.status_code,
                response.content,
                file=stderr,
            )
            return
        return (
            response.text
            if self._output == Output.TEXT
            else types.ScoreboardDeleteResponse(**response.json())
        )


class API:
    achievement: Achievement
    participant: Participant
    scoreboard: Scoreboard
    verbose: bool

    def __init__(
        self,
        api_key: Optional[str] = None,
        tags: Set[str] = set(),
        client: Optional[Client] = None,
        output: Output = Output.OBJECT,
        verbose: bool = False,
    ):
        client = client or Client(client_config=ClientConfig(api_key=api_key, tags=tags))
        self.achievement = Achievement(client=client, output=output, verbose=verbose)
        self.participant = Participant(client=client, output=output, verbose=verbose)
        self.scoreboard = Scoreboard(client=client, output=output, verbose=verbose)
        self.verbose = verbose

    def api_version(self) -> str:
        """Returns the API version."""
        return "1.0.3"

    def client_version(self) -> str:
        """Returns the client version."""
        return "0.0.2"




#!/usr/bin/env python

# Copyright 2023 Monster Street Systems LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# NB: This is a generated file; any changes will be lost.

from datetime import datetime
from random import choice

from faker import Faker
from requests import post

from vements import api, client, config, types


class Setup:
    def __init__(self):
        tags = {"development", "host", "substitute"}

        cc = client.ClientConfig(api_key="has to be something to skip env check", tags=tags)
        print("server url:", cc.server_url)

        try:
            self.db = post(cc.server_url + "-/database").json()
        except Exception as e:
            import sys

            print("could not read test db:", e, file=sys.stderr)
            sys.exit(1)

        api_key = [k for k in self.db["api_key"] if k["capability"] == "rw"][0]
        api_key_v = f"{api_key['project_id']}:{api_key['api_key_id']}"
        print("api key:", api_key_v)

        self.client = client.Client(client_config=client.ClientConfig(api_key=api_key_v, tags=tags))
        self.api = api.API(client=self.client, output=api.Output.OBJECT)
        self.fake = Faker()

    def get_value(self, key):
        if key == "recorded":
            return f"{datetime.utcnow().isoformat()}"
        project_id = self.client.api_key.split(":")[0]
        if key == "project_id":
            return project_id
        resource = key.split("_")[0]
        resources = [r for r in self.db[resource] if r["project_id"] == project_id]
        return choice(resources)[f"{resource}_id"] if resources else None

    def get_random(self, key):
        if key == "display":
            return self.fake.name()
        if key == "rank_dir":
            return choice(["asc", "desc"])
        if key == "public":
            return choice([False, True])
        if key == "external_id":
            return self.fake.bban()
        if key == "image":
            return ""
        if key == "extra":
            return dict()
        return None


def test_achievement_leaderboard():
    setup = Setup()

    params = {}
    params["achievement_id"] = setup.get_value("achievement_id")
    print("params:", params)

    call = setup.api.achievement.leaderboard
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementLeaderboardResponse)


def test_achievement_record():
    setup = Setup()

    params = {}
    params["achievement_id"] = setup.get_value("achievement_id")
    params["participant_id"] = setup.get_random("participant_id")
    params["value"] = setup.get_random("value")
    params["recorded"] = setup.get_random("recorded")
    print("params:", params)

    call = setup.api.achievement.record
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementProgressResponse)


def test_participant_progress():
    setup = Setup()

    params = {}
    params["participant_id"] = setup.get_value("participant_id")
    print("params:", params)

    call = setup.api.participant.progress
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantProgressResponse)


def test_participant_scores():
    setup = Setup()

    params = {}
    params["participant_id"] = setup.get_value("participant_id")
    print("params:", params)

    call = setup.api.participant.scores
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantScoresResponse)


def test_scoreboard_record():
    setup = Setup()

    params = {}
    params["scoreboard_id"] = setup.get_value("scoreboard_id")
    params["participant_id"] = setup.get_random("participant_id")
    params["value"] = setup.get_random("value")
    params["recorded"] = setup.get_random("recorded")
    print("params:", params)

    call = setup.api.scoreboard.record
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardScoreResponse)


def test_scoreboard_scores():
    setup = Setup()

    params = {}
    params["scoreboard_id"] = setup.get_value("scoreboard_id")
    params["from_"] = None
    params["to"] = None
    print("params:", params)

    call = setup.api.scoreboard.scores
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardScoresResponse)


def test_achievement_list():
    setup = Setup()

    params = {}
    params["project_id"] = setup.get_value("project_id")
    params["limit"] = None
    params["offset"] = None
    print("params:", params)

    call = setup.api.achievement.list
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementListResponse)


def test_achievement_create():
    setup = Setup()

    params = {}
    params["project_id"] = setup.get_value("project_id")
    params["display"] = setup.get_random("display")
    params["goal"] = setup.get_random("goal")
    params["repeats"] = setup.get_random("repeats")
    params["locked_image"] = setup.get_random("locked_image")
    params["unlocked_image"] = setup.get_random("unlocked_image")
    params["position"] = setup.get_random("position")
    params["public"] = setup.get_random("public")
    params["extra"] = setup.get_random("extra")
    print("params:", params)

    call = setup.api.achievement.create
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementCreateResponse)


def test_achievement_read():
    setup = Setup()

    params = {}
    params["achievement_id"] = setup.get_value("achievement_id")
    print("params:", params)

    call = setup.api.achievement.read
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementReadResponse)


def test_achievement_update():
    setup = Setup()

    params = {}
    params["achievement_id"] = setup.get_value("achievement_id")
    params["display"] = setup.get_random("display")
    params["goal"] = setup.get_random("goal")
    params["repeats"] = setup.get_random("repeats")
    params["locked_image"] = setup.get_random("locked_image")
    params["unlocked_image"] = setup.get_random("unlocked_image")
    params["position"] = setup.get_random("position")
    params["public"] = setup.get_random("public")
    params["extra"] = setup.get_random("extra")
    print("params:", params)

    call = setup.api.achievement.update
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementUpdateResponse)


def test_achievement_delete():
    setup = Setup()

    params = {}
    params["achievement_id"] = setup.get_value("achievement_id")
    print("params:", params)

    call = setup.api.achievement.delete
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.AchievementDeleteResponse)


def test_participant_list():
    setup = Setup()

    params = {}
    params["project_id"] = setup.get_value("project_id")
    params["limit"] = None
    params["offset"] = None
    print("params:", params)

    call = setup.api.participant.list
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantListResponse)


def test_participant_create():
    setup = Setup()

    params = {}
    params["project_id"] = setup.get_value("project_id")
    params["display"] = setup.get_random("display")
    params["external_id"] = setup.get_random("external_id")
    params["image"] = setup.get_random("image")
    params["extra"] = setup.get_random("extra")
    print("params:", params)

    call = setup.api.participant.create
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantCreateResponse)


def test_participant_read():
    setup = Setup()

    params = {}
    params["participant_id"] = setup.get_value("participant_id")
    print("params:", params)

    call = setup.api.participant.read
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantReadResponse)


def test_participant_update():
    setup = Setup()

    params = {}
    params["participant_id"] = setup.get_value("participant_id")
    params["display"] = setup.get_random("display")
    params["external_id"] = setup.get_random("external_id")
    params["image"] = setup.get_random("image")
    params["extra"] = setup.get_random("extra")
    print("params:", params)

    call = setup.api.participant.update
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantUpdateResponse)


def test_participant_delete():
    setup = Setup()

    params = {}
    params["participant_id"] = setup.get_value("participant_id")
    print("params:", params)

    call = setup.api.participant.delete
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ParticipantDeleteResponse)


def test_scoreboard_list():
    setup = Setup()

    params = {}
    params["project_id"] = setup.get_value("project_id")
    params["limit"] = None
    params["offset"] = None
    print("params:", params)

    call = setup.api.scoreboard.list
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardListResponse)


def test_scoreboard_create():
    setup = Setup()

    params = {}
    params["project_id"] = setup.get_value("project_id")
    params["display"] = setup.get_random("display")
    params["rank_dir"] = setup.get_random("rank_dir")
    params["public"] = setup.get_random("public")
    params["extra"] = setup.get_random("extra")
    print("params:", params)

    call = setup.api.scoreboard.create
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardCreateResponse)


def test_scoreboard_read():
    setup = Setup()

    params = {}
    params["scoreboard_id"] = setup.get_value("scoreboard_id")
    print("params:", params)

    call = setup.api.scoreboard.read
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardReadResponse)


def test_scoreboard_update():
    setup = Setup()

    params = {}
    params["scoreboard_id"] = setup.get_value("scoreboard_id")
    params["display"] = setup.get_random("display")
    params["rank_dir"] = setup.get_random("rank_dir")
    params["public"] = setup.get_random("public")
    params["extra"] = setup.get_random("extra")
    print("params:", params)

    call = setup.api.scoreboard.update
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardUpdateResponse)


def test_scoreboard_delete():
    setup = Setup()

    params = {}
    params["scoreboard_id"] = setup.get_value("scoreboard_id")
    print("params:", params)

    call = setup.api.scoreboard.delete
    result = call(**params)
    print("result:", result)

    assert result
    assert isinstance(result, types.ScoreboardDeleteResponse)



#!/usr/bin/env python
from typing import Optional

from fire import Fire

from vements.api import API, Output


class CLI(API):
    def __init__(self, api_key: Optional[str] = None, verbose: bool = False):
        super().__init__(api_key=api_key, output=Output.TEXT, verbose=verbose)


if __name__ == "__main__":
    Fire(CLI)



#!/usr/bin/env python

# Copyright 2023 Monster Street Systems LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from os import environ
from requests import Response, delete, get, post, put
from typing import Optional, Set

from vements.config import Config


class ClientConfig:
    api_key: str
    config: Config
    server_url: str

    def __init__(
        self,
        api_key: Optional[str] = None,
        tags: Set[str] = set(),
        config: Optional[Config] = None,
    ):
        self.api_key = str(api_key or "") or environ["API_KEY"]
        self.config = config or Config()
        env_tags = set()
        if environ.get("SERVER_TAGS", None):
            env_tags = set(environ["SERVER_TAGS"].split(","))
        tags = (
            set(tags)
            or env_tags
            or {
                "production",
            }
        )
        server_url = self.config.server_url(tags)
        if not server_url:
            raise NotImplementedError("no server_url available; tags: " + str(tags))
        self.server_url = server_url


class Client:
    _client_config: ClientConfig

    def __init__(self, client_config: ClientConfig):
        self._client_config = client_config

    @property
    def api_key(self) -> str:
        return self._client_config.api_key

    @property
    def config(self) -> Config:
        return self._client_config.config

    @property
    def headers(self) -> dict:
        return {"x-api-key": self.api_key}

    def url(self, path) -> str:
        return self._client_config.server_url + path[1:]

    def put(self, path, params) -> Response:
        return put(self.url(path), json=params, headers=self.headers)

    def get(self, path, params=None) -> Response:
        return get(self.url(path), params=params or {}, headers=self.headers)

    def post(self, path, params) -> Response:
        return post(self.url(path), json=params, headers=self.headers)

    def delete(self, path) -> Response:
        return delete(self.url(path), headers=self.headers)



#!/usr/bin/env python

# Copyright 2023 Monster Street Systems LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# NB: This is a generated file; any changes will be lost.

from typing import Optional, Set


class Contact:
    name: str
    url: str
    email: str

    def __init__(self, name: str, url: str, email: str):
        self.name = name
        self.url = url
        self.email = email


class License:
    name: str
    url: str

    def __init__(self, name: str, url: str):
        self.name = name
        self.url = url


class ExternalDocs:
    description: str
    url: str

    def __init__(self, description: str, url: str):
        self.description = description
        self.url = url


class Server:
    url: str
    description: str
    variables: dict[str, str]
    tags: Set[str]

    def __init__(self, url: str, description: str, variables: dict[str, str], tags: Set[str]):
        self.url = url
        self.description = description
        self.variables = variables
        self.tags = tags


class Config:
    title: str
    version: str
    description: str
    terms_of_service: str
    contact: Contact
    license: License
    external_docs: ExternalDocs
    servers: Set[Server]

    def __init__(
        self,
        title: str = "Vements REST API",
        version: str = "1.0.3",
        description: str = "This specification describes the Vements REST API, its endpoints, and  the data structures used to communicate with it.",
        terms_of_service: str = "https://vements.io/terms",
        contact: Contact = Contact(
            name="Vements Support Contact",
            url="https://vements.io",
            email="support@vements.io",
        ),
        license: License = License(
            name="MIT",
            url="https://opensource.org/license/mit/",
        ),
        external_docs: ExternalDocs = ExternalDocs(
            description="Vements REST API Documentation",
            url="https://vements.io/docs",
        ),
        servers: Set[Server] = {
            Server(
                url="https://a.vements.io/{basePath}",
                description="Production Server",
                variables={
                    "basePath": "api/rest/v1.0.3/",
                },
                tags={
                    "production",
                },
            ),
            Server(
                url="http://api.localtest.me/{basePath}",
                description="Development Server (Host)",
                variables={
                    "basePath": "api/rest/v1.0.3/",
                },
                tags={
                    "development",
                    "host",
                    "full",
                },
            ),
            Server(
                url="http://localhost:9000/{basePath}",
                description="Development Server (Host Substitute)",
                variables={
                    "basePath": "api/rest/v1.0.3/",
                },
                tags={
                    "development",
                    "host",
                    "substitute",
                },
            ),
            Server(
                url="http://api-server-a:8080/{basePath}",
                description="Development Server (Container)",
                variables={
                    "basePath": "api/rest/v1.0.3/",
                },
                tags={
                    "development",
                    "container",
                    "full",
                },
            ),
            Server(
                url="http://substitute-server:9000/{basePath}",
                description="Development Server (Container Substitute)",
                variables={
                    "basePath": "api/rest/v1.0.3/",
                },
                tags={
                    "development",
                    "container",
                    "substitute",
                },
            ),
        },
    ):
        self.title = title
        self.version = version
        self.description = description
        self.terms_of_service = terms_of_service
        self.contact = contact
        self.license = license
        self.external_docs = external_docs
        self.servers = servers

    def server_url(self, matching: Set[str]) -> Optional[str]:
        servers = [server for server in self.servers if server.tags >= matching]
        if not servers:
            return
        url = servers[0].url
        for key, value in servers[0].variables.items():
            url = url.replace("{" + key + "}", value)
        return url




#!/usr/bin/env python

# Copyright 2023 Monster Street Systems LLC
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# NB: This is a generated file; any changes will be lost.

from datetime import datetime
from typing import List, Optional

from ulid import ULID


class User:
    user_id: str
    email: str
    display: str
    created: datetime
    updated: datetime
    deleted: Optional[datetime]

    def __init__(
        self,
        user_id: str,
        email: str,
        display: str,
        created: Optional[datetime] = None,
        updated: Optional[datetime] = None,
        deleted: Optional[datetime] = None,
    ):
        self.user_id = user_id
        self.email = email
        self.display = display
        self.created = created or datetime.now()
        self.updated = updated or datetime.now()
        self.deleted = deleted


class Project:
    project_id: ULID | str
    user_id: str
    display: str
    created: datetime
    updated: datetime
    deleted: Optional[datetime]
    extra: Optional[object]

    def __init__(
        self,
        project_id: ULID | str,
        user_id: str,
        display: str,
        created: Optional[datetime] = None,
        updated: Optional[datetime] = None,
        deleted: Optional[datetime] = None,
        extra: Optional[object] = None,
    ):
        self.project_id = project_id
        self.user_id = user_id
        self.display = display
        self.created = created or datetime.now()
        self.updated = updated or datetime.now()
        self.deleted = deleted
        self.extra = extra


class ApiKey:
    api_key_id: str
    project_id: ULID | str
    display: str
    capability: str
    deactivated: Optional[datetime]
    last_used: Optional[datetime]
    created: datetime
    updated: datetime
    deleted: Optional[datetime]

    def __init__(
        self,
        api_key_id: str,
        project_id: ULID | str,
        display: str,
        capability: str,
        created: Optional[datetime] = None,
        updated: Optional[datetime] = None,
        deactivated: Optional[datetime] = None,
        last_used: Optional[datetime] = None,
        deleted: Optional[datetime] = None,
    ):
        self.api_key_id = api_key_id
        self.project_id = project_id
        self.display = display
        self.capability = capability
        self.deactivated = deactivated
        self.last_used = last_used
        self.created = created or datetime.now()
        self.updated = updated or datetime.now()
        self.deleted = deleted


class Achievement:
    achievement_id: ULID | str
    project_id: ULID | str
    display: str
    goal: int
    repeats: int
    locked_image: Optional[str]
    unlocked_image: Optional[str]
    position: int
    public: bool
    created: datetime
    updated: datetime
    deleted: Optional[datetime]
    extra: Optional[object]

    def __init__(
        self,
        achievement_id: ULID | str,
        project_id: ULID | str,
        display: str,
        goal: int,
        repeats: int,
        position: int,
        public: bool = False,
        created: Optional[datetime] = None,
        updated: Optional[datetime] = None,
        locked_image: Optional[str] = "",
        unlocked_image: Optional[str] = "",
        deleted: Optional[datetime] = None,
        extra: Optional[object] = None,
    ):
        self.achievement_id = achievement_id
        self.project_id = project_id
        self.display = display
        self.goal = goal
        self.repeats = repeats
        self.locked_image = locked_image
        self.unlocked_image = unlocked_image
        self.position = position
        self.public = public
        self.created = created or datetime.now()
        self.updated = updated or datetime.now()
        self.deleted = deleted
        self.extra = extra


class Participant:
    participant_id: ULID | str
    project_id: ULID | str
    display: str
    external_id: str
    image: Optional[str]
    created: datetime
    updated: datetime
    deleted: Optional[datetime]
    extra: Optional[object]

    def __init__(
        self,
        participant_id: ULID | str,
        project_id: ULID | str,
        display: str,
        external_id: str,
        created: Optional[datetime] = None,
        updated: Optional[datetime] = None,
        image: Optional[str] = "",
        deleted: Optional[datetime] = None,
        extra: Optional[object] = None,
    ):
        self.participant_id = participant_id
        self.project_id = project_id
        self.display = display
        self.external_id = external_id
        self.image = image
        self.created = created or datetime.now()
        self.updated = updated or datetime.now()
        self.deleted = deleted
        self.extra = extra


class Scoreboard:
    scoreboard_id: ULID | str
    project_id: ULID | str
    display: str
    rank_dir: str
    public: bool
    created: datetime
    updated: datetime
    deleted: Optional[datetime]
    extra: Optional[object]

    def __init__(
        self,
        scoreboard_id: ULID | str,
        project_id: ULID | str,
        display: str,
        rank_dir: str,
        public: bool = False,
        created: Optional[datetime] = None,
        updated: Optional[datetime] = None,
        deleted: Optional[datetime] = None,
        extra: Optional[object] = None,
    ):
        self.scoreboard_id = scoreboard_id
        self.project_id = project_id
        self.display = display
        self.rank_dir = rank_dir
        self.public = public
        self.created = created or datetime.now()
        self.updated = updated or datetime.now()
        self.deleted = deleted
        self.extra = extra


class Progress:
    progress_id: int
    achievement_id: ULID | str
    participant_id: ULID | str
    value: int
    recorded: Optional[datetime]

    def __init__(
        self,
        progress_id: int,
        achievement_id: ULID | str,
        participant_id: ULID | str,
        value: int,
        recorded: Optional[datetime] = None,
    ):
        self.progress_id = progress_id
        self.achievement_id = achievement_id
        self.participant_id = participant_id
        self.value = value
        self.recorded = recorded


class Score:
    score_id: int
    scoreboard_id: ULID | str
    participant_id: ULID | str
    value: int
    recorded: Optional[datetime]

    def __init__(
        self,
        score_id: int,
        scoreboard_id: ULID | str,
        participant_id: ULID | str,
        value: int,
        recorded: Optional[datetime] = None,
    ):
        self.score_id = score_id
        self.scoreboard_id = scoreboard_id
        self.participant_id = participant_id
        self.value = value
        self.recorded = recorded


class AchievementLeaderboardItem:
    participant: Participant
    progress: int
    iterations: int

    def __init__(
        self,
        participant: Participant,
        progress: int,
        iterations: int,
    ):
        self.participant = participant
        self.progress = progress
        self.iterations = iterations


class ParticipantProgressItem:
    achievement: Achievement
    progress: int
    iterations: int

    def __init__(
        self,
        achievement: Achievement,
        progress: int,
        iterations: int,
    ):
        self.achievement = achievement
        self.progress = progress
        self.iterations = iterations


class ParticipantScoreItem:
    scoreboard: Scoreboard
    rank: int
    total: int

    def __init__(
        self,
        scoreboard: Scoreboard,
        rank: int,
        total: int,
    ):
        self.scoreboard = scoreboard
        self.rank = rank
        self.total = total


class ScoreboardScoreItem:
    participant_id: ULID | str
    participant: Participant
    rank: int
    total: int

    def __init__(
        self,
        participant_id: ULID | str,
        participant: Participant,
        rank: int,
        total: int,
    ):
        self.participant_id = participant_id
        self.participant = participant
        self.rank = rank
        self.total = total


class AchievementLeaderboardResponse:
    achievement_leaderboard: List[AchievementLeaderboardItem]

    def __init__(
        self,
        achievement_leaderboard: List[AchievementLeaderboardItem],
    ):
        self.achievement_leaderboard = achievement_leaderboard


class AchievementProgressResponse:
    insert_progress_one: Progress

    def __init__(
        self,
        insert_progress_one: Progress,
    ):
        self.insert_progress_one = insert_progress_one


class ParticipantProgressResponse:
    participant_progress: List[ParticipantProgressItem]

    def __init__(
        self,
        participant_progress: List[ParticipantProgressItem],
    ):
        self.participant_progress = participant_progress


class ParticipantScoresResponse:
    participant_scores: List[ParticipantScoreItem]

    def __init__(
        self,
        participant_scores: List[ParticipantScoreItem],
    ):
        self.participant_scores = participant_scores


class ScoreboardScoreResponse:
    insert_score_one: Score

    def __init__(
        self,
        insert_score_one: Score,
    ):
        self.insert_score_one = insert_score_one


class ScoreboardScoresResponse:
    scoreboard_scores: List[ScoreboardScoreItem]

    def __init__(
        self,
        scoreboard_scores: List[ScoreboardScoreItem],
    ):
        self.scoreboard_scores = scoreboard_scores


class AchievementProgressRequest:
    participant_id: ULID | str
    value: int
    recorded: Optional[datetime]

    def __init__(
        self,
        participant_id: ULID | str,
        value: int,
        recorded: Optional[datetime] = None,
    ):
        self.participant_id = participant_id
        self.value = value
        self.recorded = recorded


class ScoreboardScoreRequest:
    participant_id: ULID | str
    value: int
    recorded: Optional[datetime]

    def __init__(
        self,
        participant_id: ULID | str,
        value: int,
        recorded: Optional[datetime] = None,
    ):
        self.participant_id = participant_id
        self.value = value
        self.recorded = recorded


class AchievementCreateRequest:
    project_id: ULID | str
    display: str
    goal: int
    repeats: int
    locked_image: Optional[str]
    unlocked_image: Optional[str]
    position: int
    public: bool
    extra: Optional[object]

    def __init__(
        self,
        project_id: ULID | str,
        display: str,
        goal: int,
        repeats: int,
        position: int,
        public: bool = False,
        locked_image: Optional[str] = "",
        unlocked_image: Optional[str] = "",
        extra: Optional[object] = None,
    ):
        self.project_id = project_id
        self.display = display
        self.goal = goal
        self.repeats = repeats
        self.locked_image = locked_image
        self.unlocked_image = unlocked_image
        self.position = position
        self.public = public
        self.extra = extra


class AchievementCreateResponse:
    insert_achievement_one: Achievement

    def __init__(
        self,
        insert_achievement_one: Achievement,
    ):
        self.insert_achievement_one = insert_achievement_one


class AchievementReadResponse:
    achievement: List[Achievement]

    def __init__(
        self,
        achievement: List[Achievement],
    ):
        self.achievement = achievement


class AchievementUpdateRequest:
    display: str
    goal: int
    repeats: int
    locked_image: Optional[str]
    unlocked_image: Optional[str]
    position: int
    public: bool
    extra: Optional[object]

    def __init__(
        self,
        display: str,
        goal: int,
        repeats: int,
        position: int,
        public: bool = False,
        locked_image: Optional[str] = "",
        unlocked_image: Optional[str] = "",
        extra: Optional[object] = None,
    ):
        self.display = display
        self.goal = goal
        self.repeats = repeats
        self.locked_image = locked_image
        self.unlocked_image = unlocked_image
        self.position = position
        self.public = public
        self.extra = extra


class AchievementUpdateResponse:
    update_achievement_by_pk: Achievement

    def __init__(
        self,
        update_achievement_by_pk: Achievement,
    ):
        self.update_achievement_by_pk = update_achievement_by_pk


class AchievementDeleteResponse:
    delete_achievement_by_pk: Optional[Achievement]

    def __init__(
        self,
        delete_achievement_by_pk: Optional[Achievement] = None,
    ):
        self.delete_achievement_by_pk = delete_achievement_by_pk


class AchievementListResponse:
    achievement: List[Achievement]

    def __init__(
        self,
        achievement: List[Achievement],
    ):
        self.achievement = achievement


class ParticipantCreateRequest:
    project_id: ULID | str
    display: str
    external_id: str
    image: Optional[str]
    extra: Optional[object]

    def __init__(
        self,
        project_id: ULID | str,
        display: str,
        external_id: str,
        image: Optional[str] = "",
        extra: Optional[object] = None,
    ):
        self.project_id = project_id
        self.display = display
        self.external_id = external_id
        self.image = image
        self.extra = extra


class ParticipantCreateResponse:
    insert_participant_one: Participant

    def __init__(
        self,
        insert_participant_one: Participant,
    ):
        self.insert_participant_one = insert_participant_one


class ParticipantReadResponse:
    participant: List[Participant]

    def __init__(
        self,
        participant: List[Participant],
    ):
        self.participant = participant


class ParticipantUpdateRequest:
    display: str
    external_id: str
    image: Optional[str]
    extra: Optional[object]

    def __init__(
        self,
        display: str,
        external_id: str,
        image: Optional[str] = "",
        extra: Optional[object] = None,
    ):
        self.display = display
        self.external_id = external_id
        self.image = image
        self.extra = extra


class ParticipantUpdateResponse:
    update_participant_by_pk: Participant

    def __init__(
        self,
        update_participant_by_pk: Participant,
    ):
        self.update_participant_by_pk = update_participant_by_pk


class ParticipantDeleteResponse:
    delete_participant_by_pk: Optional[Participant]

    def __init__(
        self,
        delete_participant_by_pk: Optional[Participant] = None,
    ):
        self.delete_participant_by_pk = delete_participant_by_pk


class ParticipantListResponse:
    participant: List[Participant]

    def __init__(
        self,
        participant: List[Participant],
    ):
        self.participant = participant


class ScoreboardCreateRequest:
    project_id: ULID | str
    display: str
    rank_dir: str
    public: bool
    extra: Optional[object]

    def __init__(
        self,
        project_id: ULID | str,
        display: str,
        rank_dir: str,
        public: bool = False,
        extra: Optional[object] = None,
    ):
        self.project_id = project_id
        self.display = display
        self.rank_dir = rank_dir
        self.public = public
        self.extra = extra


class ScoreboardCreateResponse:
    insert_scoreboard_one: Scoreboard

    def __init__(
        self,
        insert_scoreboard_one: Scoreboard,
    ):
        self.insert_scoreboard_one = insert_scoreboard_one


class ScoreboardReadResponse:
    scoreboard: List[Scoreboard]

    def __init__(
        self,
        scoreboard: List[Scoreboard],
    ):
        self.scoreboard = scoreboard


class ScoreboardUpdateRequest:
    display: str
    rank_dir: str
    public: bool
    extra: Optional[object]

    def __init__(
        self,
        display: str,
        rank_dir: str,
        public: bool = False,
        extra: Optional[object] = None,
    ):
        self.display = display
        self.rank_dir = rank_dir
        self.public = public
        self.extra = extra


class ScoreboardUpdateResponse:
    update_scoreboard_by_pk: Scoreboard

    def __init__(
        self,
        update_scoreboard_by_pk: Scoreboard,
    ):
        self.update_scoreboard_by_pk = update_scoreboard_by_pk


class ScoreboardDeleteResponse:
    delete_scoreboard_by_pk: Optional[Scoreboard]

    def __init__(
        self,
        delete_scoreboard_by_pk: Optional[Scoreboard] = None,
    ):
        self.delete_scoreboard_by_pk = delete_scoreboard_by_pk


class ScoreboardListResponse:
    scoreboard: List[Scoreboard]

    def __init__(
        self,
        scoreboard: List[Scoreboard],
    ):
        self.scoreboard = scoreboard

